@@ -34,7 +34,7 @@ static guint font_height=0;
 
 enum { VIEW_CHANGED_SIGNAL, SELECTION_CHANGED_SIGNAL, CURSOR_CHANGED_SIGNAL, 
        CHUNK_CHANGED_SIGNAL, DOUBLE_CLICK_SIGNAL, LAST_SIGNAL };
-static gint chunk_view_signals[LAST_SIGNAL] = { 0 };
+static guint chunk_view_signals[LAST_SIGNAL] = { 0 };
 
 static void chunk_view_update_image(ChunkView *view, guint xs, guint xe);
 static void chunk_view_redraw_samples(ChunkView *cv, off_t start, off_t end);
@@ -496,7 +496,12 @@ static gint scroll_wheel(GtkWidget *widget, gdouble mouse_x, int direction)
      ui2 = dragstart+(off_t)((gfloat)(d->viewend-dragstart))*zf;
      if (ui1 < 0) ui1 = 0;
      if (ui2 < dragstart || ui2 > d->chunk->length) ui2 = d->chunk->length;
-     if (ui2 == ui1) ui2++;
+     if (ui2 == ui1) {
+	  if (ui2 < d->chunk->length)
+	       ui2++;
+	  else if (ui1 > 0)
+	       ui1--;
+     }
      document_set_view(d,ui1,ui2);
      return FALSE;
 }
@@ -668,7 +673,7 @@ gboolean chunk_view_autoscroll(void)
 static gint chunk_view_button_release(GtkWidget *widget, GdkEventButton *event)
 {
      ChunkView *cv = CHUNKVIEW(widget);
-     if (event->button == 2) {
+     if (event->button == 2 && cv->doc != NULL) {
 	  document_play_selection(cv->doc,FALSE,1.0);
      }
      autoscroll = FALSE;
@@ -722,7 +727,7 @@ static void chunk_view_init(GtkObject *obj)
 #else
 	  PangoLayout *pl;
 	  pl = gtk_widget_create_pango_layout( GTK_WIDGET(obj), "0123456789" );
-	  pango_layout_get_pixel_size(pl, NULL, &font_height);
+	  pango_layout_get_pixel_size(pl, NULL, (gint *)&font_height);
 	  g_object_unref(pl);
 #endif
      }
