@@ -58,6 +58,7 @@ struct file_type {
 };
 
 static GList *file_types = NULL;
+static struct file_type *raw_type,*mplayer_type;
 
 static gboolean wav_check(gchar *filename);
 static Chunk *wav_load(gchar *filename, int dither_mode, StatusBar *bar);
@@ -176,8 +177,10 @@ static void setup_types(void)
 	    t->get_settings = mp3_get_settings;
 	    t->free_settings = g_free;
      }
-     register_file_type(_("Raw PCM data"), ".raw", FALSE,NULL, raw_load, 
-			raw_save, 0);
+     raw_type = register_file_type(_("Raw PCM data"), ".raw", FALSE,NULL, 
+				   raw_load, raw_save, 0);
+     mplayer_type = register_file_type(_("Open with MPlayer"), NULL,TRUE,NULL,
+				       try_mplayer, NULL, 0);
 }
 
 guint fileformat_count(void)
@@ -242,8 +245,12 @@ static Chunk *chunk_load_main(gchar *filename, int dither_mode, StatusBar *bar,
      }
      /* Try mplayer if available */
      chunk = try_mplayer(filename,dither_mode,bar);
-     if (chunk != NULL) return chunk;
+     if (chunk != NULL) {
+	  *format = mplayer_type;
+	  return chunk;
+     }
      /* Use the raw loader */
+     *format = raw_type;
      return raw_load(filename,dither_mode,bar);
 }
 
@@ -253,7 +260,7 @@ Chunk *chunk_load_x(gchar *filename, int dither_mode, StatusBar *bar,
      Chunk *chunk;
      gchar *c;
      EFILE *f;
-     struct file_type *ft;
+     struct file_type *ft = NULL;
 
      /* First, see if the file exists */
      if (!file_exists(filename)) {
@@ -525,12 +532,13 @@ static Chunk *wav_load(char *filename, int dither_mode, StatusBar *bar)
 static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
 			 struct file_type *type, int dither_mode, 
 			 StatusBar *bar, gboolean *fatal)
-{
+{    
+     Chunk *c;
      EFILE *f;
      Datasource *ds;
      DataPart *dp;
-     Dataformat *fmt;
-     gboolean b;
+     Dataformat *fmt,cfmt;
+     gboolean b,q;
 
      fmt = &(chunk->format);
      
@@ -546,17 +554,34 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
      }
 #endif
 
+     if (fmt->type == DATAFORMAT_PCM) {
+	  /* Check that the sign and endian-ness is correct */
+	  if (fmt->samplesize == 1) q = FALSE;
+	  else q = TRUE;
+	  if (XOR(fmt->sign,q) || fmt->bigendian) {
+	       memcpy(&cfmt,fmt,sizeof(cfmt));
+	       cfmt.sign = q;
+	       cfmt.bigendian = FALSE;
+	       c = chunk_convert(chunk,&cfmt,DITHER_UNSPEC,bar);
+	       b = wav_save(c,filename,settings,type,dither_mode,bar,fatal);
+	       gtk_object_sink(GTK_OBJECT(c));
+	       return b;
+	  }
+     }
+
+     
      if (fmt->type == DATAFORMAT_PCM && fmt->samplesize==1 && fmt->sign) {
-	  user_error(_("8-bit wav-files must be in unsigned format!"));
-	  return TRUE;
+	  /* user_error(_("8-bit wav-files must be in unsigned format!")); */
+	  g_assert_not_reached();
      }
      if (fmt->type == DATAFORMAT_PCM && fmt->samplesize>1 && !fmt->sign) {
-	  user_error(_("16/24/32-bit wav-files must be in signed format!"));
-	  return TRUE;
+	  /*user_error(_("16/24/32-bit wav-files must be in signed format!"));
+	  */
+	  g_assert_not_reached();
      }    
      if (fmt->type == DATAFORMAT_PCM && fmt->bigendian == TRUE) {
-	  user_error(_("wav files must be in little endian format!"));
-	  return TRUE;
+	  /* user_error(_("wav files must be in little endian format!")); */
+	  g_assert_not_reached();
      }
 
      /* Give a warning once if we're saving a file larger than 2GB */
@@ -624,17 +649,18 @@ static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar)
      Datasource *ds;
      Dataformat *fmt;
      off_t i;
-     fmt = rawdialog_execute(filename);
-     if (!fmt) return NULL;
+     guint offs;
      i = errdlg_filesize(filename);
      if (i==-1) return NULL;
+     fmt = rawdialog_execute(filename,i,&offs);
+     if (!fmt) return NULL;
      ds = (Datasource *)gtk_type_new(datasource_get_type());
      memcpy(&(ds->format),fmt,sizeof(Dataformat));
-     ds->bytes = i;
-     ds->length = i/fmt->samplebytes;
+     ds->bytes = i-offs;
+     ds->length = (i-offs)/fmt->samplebytes;
      ds->type = DATASOURCE_VIRTUAL;
      ds->data.virtual.filename = g_strdup(filename);
-     ds->data.virtual.offset = 0;
+     ds->data.virtual.offset = offs;
      return chunk_new_from_datasource(ds);
 }
 
@@ -690,6 +716,7 @@ static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar)
      f.type = DATAFORMAT_PCM;
      f.samplerate = info.samplerate;
      f.channels = info.channels;
+     f.bigendian = IS_BIGENDIAN;
      /* Fix samplesize parameter */
      switch (info.format&SF_FORMAT_SUBMASK) {
      case SF_FORMAT_PCM_U8: f.sign=FALSE;f.samplesize=1;raw_readable=TRUE;break;
