@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2007, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -94,6 +94,12 @@ int main(int argc, char **argv)
      gboolean b;
      gchar *c;
 
+     /* Some versions of the aRts C library call g_thread_init internally,
+      * and that confuses GLib2's memory allocation stuff. */
+#ifdef HAVE_GTHREAD
+     g_thread_init(NULL);
+#endif
+
      /* Set default locale */
      setlocale(LC_ALL,"");
 
@@ -193,7 +199,7 @@ int main(int argc, char **argv)
      set_custom_colors(NULL);
 
      /* Register effects */
-     effect_browser_register_default_effects();
+     effect_register_init();
      if (ladspa) ladspa_dialog_register();
      sox_dialog_register();
 
@@ -360,7 +366,7 @@ GdkGC *get_gc(enum Color c, GtkWidget *w)
      return gc_table[c];
 }
 
-static int hexval(gchar chr)
+int hexval(gchar chr)
 {
      if (chr >= '0' && chr <= '9') return chr-'0';
      else if (chr >= 'a' && chr <= 'f') return chr-'a'+10;
