@@ -41,6 +41,7 @@
 #define GTK_CLASS_TYPE(klass) (((GtkObjectClass *)klass)->type)
 #define gtk_style_get_font(style) ((style)->font)
 #define GTK_WIN_POS_CENTER_ON_PARENT GTK_WIN_POS_CENTER
+#define gtk_container_get_children gtk_container_children
 #else
 #define GTK_WINDOW_DIALOG GTK_WINDOW_TOPLEVEL
 #define gtk_object_class_add_signals(x,y,z)
@@ -105,6 +106,8 @@ gchar *get_home_directory(void);
 
 void do_yield(gboolean may_sleep);
 
+int hexval(gchar chr);
+
 void launch_mixer(void);
 
 GtkLabel *attach_label(gchar *text, GtkWidget *table, guint y, guint x);
