@@ -81,6 +81,8 @@ static Chunk *clipboard = NULL;
 gboolean autoplay_mark_flag = FALSE;
 gboolean varispeed_reset_flag = FALSE;
 
+static void mainwindow_set_document(Mainwindow *w, Document *doc, 
+				    gchar *filename);
 static void mainwindow_view_changed(Document *d, gpointer user_data);
 static void mainwindow_selection_changed(Document *d, gpointer user_data);
 static void mainwindow_cursor_changed(Document *d, gboolean rolling, 
@@ -250,6 +252,16 @@ static void typesel_changed(Combo *obj, GtkWidget *user_data)
      }
 }
 
+static void mainwindow_open(Mainwindow *w, gchar *filename)
+{
+     Document *doc;
+     doc = document_new_with_file ( filename, w->statusbar );
+     if (doc == NULL) return;
+     mainwindow_set_document ( w, doc, filename );
+     inifile_set("lastOpenFile",filename);
+     recent_file(filename);
+}
+
 static gchar *get_save_filename(gchar *old_filename, gchar *title_text, 
 				gint *type_id, gboolean *use_defaults)
 {
@@ -600,6 +612,79 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
      return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget,event);
 }
 
+static void urldecode(char *str)
+{
+     char *s,*d;
+     int i,j;
+     s = d = str;
+     while (*s != 0) {
+	  if (*s != '%' || s[1] == 0 || s[2] == 0) {
+	       *d = *s;
+	       d ++;
+	       s ++;
+	  } else {
+	       i = hexval(s[1]);
+	       j = hexval(s[2]);
+	       if (i < 0 || j < 0) { 
+		    *d = *s;
+		    d ++;
+		    s ++;
+		    continue;
+	       } 
+	       *d = (char)(i*16+j);
+	       d ++;
+	       s += 3;
+	  }
+	  
+     }
+     *d = 0;
+}
+
+static void mainwindow_drag_data_received(GtkWidget *widget, 
+					  GdkDragContext *dc, gint x, gint y, 
+					  GtkSelectionData *selection_data, 
+					  guint info, guint t)
+{
+     gchar *c,*d,*e,*f;
+     if (selection_data->length > 0) {
+	  
+	  /* Copy and add extra null for safety */
+	  c = g_malloc(selection_data->length+1);
+	  memcpy(c,selection_data->data,selection_data->length);
+	  c[selection_data->length] = 0;
+
+	  gtk_drag_finish(dc, TRUE, FALSE, t);
+	  
+	  /* Split into files */
+	  for (d=strtok(c,"\n\r"); d!=NULL; d=strtok(NULL,"\n\r")) {
+	       e = strchr(d,':');
+	       if (e == NULL) continue;
+	       *e = 0;
+	       if (strcmp(d,"file")) continue;
+	       while (e[1] == '/') e++;
+	       
+	       urldecode(e);
+	       
+	       if (!file_exists(e)) {
+		    f = g_strdup_printf("Silently ignoring non-esisting "
+					"file: %s\n",e);
+		    console_message(f);
+		    g_free(f);
+		    continue;
+	       }
+
+	       mainwindow_open(MAINWINDOW(widget),e);
+	       /* printf("Dropped: %s\n",e); */
+	       
+	  }
+
+	  g_free(c);
+
+     } else
+
+	  gtk_drag_finish(dc, FALSE, FALSE, t);
+}
+
 static void mainwindow_class_init(GtkObjectClass *klass)
 {
      parent_class = gtk_type_class(gtk_window_get_type());
@@ -607,7 +692,8 @@ static void mainwindow_class_init(GtkObjectClass *klass)
      GTK_WIDGET_CLASS(klass)->delete_event = mainwindow_delete_event;
      GTK_WIDGET_CLASS(klass)->realize = mainwindow_realize;
      GTK_WIDGET_CLASS(klass)->key_press_event = mainwindow_keypress;     
-
+     GTK_WIDGET_CLASS(klass)->drag_data_received = 
+	  mainwindow_drag_data_received;
 }
 
 static void mainwindow_set_document(Mainwindow *w, Document *d, 
@@ -660,7 +746,6 @@ void mainwindow_set_speed_sensitive(gboolean sensitive)
 static void file_open(GtkMenuItem *menuitem, gpointer user_data)
 {
      gchar *c;
-     Document *doc;
      Mainwindow *w = MAINWINDOW ( user_data );     
      if (w->doc != NULL && w->doc->filename != NULL)
 	  c = get_filename(w->doc->filename,"*.wav",_("Load File"),FALSE,NULL);
@@ -670,14 +755,7 @@ static void file_open(GtkMenuItem *menuitem, gpointer user_data)
 	  c = get_filename(c,"*.wav", _("Load File"), FALSE, NULL);
      }
      if (!c) return;
-     doc = document_new_with_file ( c, w->statusbar );
-     if (doc == NULL) {
-	  g_free(c);
-	  return;
-     }
-     mainwindow_set_document ( w, doc, c );
-     inifile_set("lastOpenFile",c);
-     recent_file(c);
+     mainwindow_open(w,c);
      g_free(c);
 }
 
@@ -914,7 +992,7 @@ static void edit_delete(GtkMenuItem *menuitem, gpointer user_data)
 
 static Chunk *interp(Chunk *chunk, StatusBar *bar, gpointer user_data)
 {
-  return chunk_interpolate_endpoints(chunk,dither_editing,bar);
+  return chunk_interpolate_endpoints(chunk,TRUE,dither_editing,bar);
 }
 
 static void edit_silence(GtkMenuItem *menuitem, gpointer user_data)
@@ -967,12 +1045,12 @@ static void view_zoomall(GtkMenuItem *menuitem, gpointer user_data)
 #ifdef SHOW_DEBUG_MENU
 static void debug_mark(GtkMenuItem *menuitem, gpointer user_data)
 {
-     MAINWINDOW(user_data)->changed = TRUE;
+     /* MAINWINDOW(user_data)->changed = TRUE; */
 }
 
 static gboolean dummy_proc(void *in, guint sample_size, 
 			   chunk_writeout_func out_func, WriteoutID id,
-			   Chunk *source)
+			   Dataformat *format)
 {
      return out_func(id,in,sample_size);
 }
@@ -980,7 +1058,7 @@ static gboolean dummy_proc(void *in, guint sample_size,
 static void debug_dummy(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
-     mainwindow_chunk_change(w,chunk_filter(w->view->chunk,dummy_proc,NULL,CHUNK_FILTER_MANY,FALSE,w,"DUMMY"),NULL,0,0,0);
+     document_apply(w->doc,dummy_proc,NULL,CHUNK_FILTER_MANY,FALSE,"DUMMY");
 }
 
 static void checkoc_proc(Chunk *chunk, gpointer user_data)
@@ -997,14 +1075,22 @@ static void debug_checkoc(GtkMenuItem *menuitem, gpointer user_data)
 
 static void dump_format(Dataformat *df)
 {
-     printf("%dx%d@%dHz %c\n",df->channels,df->samplesize,df->samplerate,
-	    df->sign?'S':'U');
+     printf("%d Channels @ %dHz ",df->channels,df->samplerate);
+     if (df->type == DATAFORMAT_FLOAT) {
+	  if (df->samplesize > sizeof(float)) {
+	       puts("float");
+	  } else
+	       puts("double");
+     } else {
+	  printf("%d%c%c\n",df->samplesize,df->sign?'S':'U',
+		 df->bigendian?'B':'L');
+     }
 }
 
 static void dump_dp(DataPart *dp)
 {
-     printf("        Datasource@%p, L/S=%Ld/%Ld <%s> Format: ",dp->ds,
-	    dp->ds->length,dp->ds->size,dp->ds->fake_pcm?"fake":"pcm");
+     printf("        Datasource@%p, L/S=%Ld/%Ld Format: ",dp->ds,
+	    dp->ds->length,dp->ds->bytes);
      dump_format(&(dp->ds->format));
      printf("        Usage: %Ld + %Ld\n",dp->position,dp->length);
      switch (dp->ds->type) {
@@ -1032,6 +1118,9 @@ static void dump_dp(DataPart *dp)
 		 dp->ds->data.sndfile.filename, dp->ds->data.sndfile.pos,
 		 dp->ds->data.sndfile.raw_readable?'T':'F');
 	  break;
+     case DATASOURCE_REF:
+	  printf("        REF --> %p\n",dp->ds->data.clone);
+	  break;	  
      case DATASOURCE_CLONE:
 	  printf("        CLONE --> %p\n",dp->ds->data.clone);
 	  break;
@@ -1056,12 +1145,22 @@ static void indirect_dump_chunk(Chunk **cp)
 
 static void windowinfo_proc(gpointer item, gpointer user_data)
 {
+     struct HistoryEntry *he;
      Mainwindow *w = MAINWINDOW(item);     
-     printf(_("\nWindow '%s'\n"),w->filename);
+     printf(_("\nWindow '%s'\n"),w->doc->filename);
      puts(_("  Current chunk:"));
-     dump_chunk(w->view->chunk);
+     dump_chunk(w->view->doc->chunk);
      puts(_("  History:"));
-     g_slist_foreach(w->history,(GFunc)indirect_dump_chunk,NULL);
+     he = w->doc->history_pos;
+     if (he == NULL) {
+	  puts("    (empty)");
+     } else {
+	  while (he->prev != NULL) he=he->prev;
+	  while (he != NULL) {
+	       dump_chunk(he->chunk);
+	       he=he->next;
+	  }
+     }
 }
 
 static void debug_chunkinfo(GtkMenuItem *menuitem, gpointer user_data)
@@ -1074,7 +1173,9 @@ static void debug_chunkinfo(GtkMenuItem *menuitem, gpointer user_data)
 static void mainwindow_show_effect_dialog(Mainwindow *mw, gchar *effect_name)
 {
      EffectBrowser *eb;
-     eb = EFFECT_BROWSER(effect_browser_new_with_effect(mw->doc,effect_name));
+     eb = EFFECT_BROWSER(effect_browser_new_with_effect(mw->doc,effect_name,
+							'B',
+							(effect_name!=NULL)));
      gtk_widget_show(GTK_WIDGET(eb));
 }
 
@@ -1793,7 +1894,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Effects/Effects dialog..."),"<control>E",effects_dialog,0,NULL  },
 #ifdef SHOW_DEBUG_MENU
 	  { N_("/Debug"),         NULL,         NULL,           0, "<Branch>"    },
-	  { N_("/Debug/Mark as modified"),NULL, debug_mark,     0, NULL          },
+	  /*	  { N_("/Debug/Mark as modified"),NULL, debug_mark,     0, NULL          }, */
 	  { N_("/Debug/Dummy effect"),NULL,     debug_dummy,    0, NULL          },
 	  { N_("/Debug/Check opencount"),NULL,  debug_checkoc,  0, NULL          },
 	  { N_("/Debug/Dump chunk info"),NULL,  debug_chunkinfo,0, NULL          },
@@ -2270,6 +2371,7 @@ static void mainwindow_init(Mainwindow *obj)
      GdkPixmap *p;
      GdkBitmap *mask;
      GtkRequisition req;
+     GtkTargetEntry gte;
 
      if (!window_geometry_stack_inited) {
 	  if (inifile_get_gboolean("useGeometry",FALSE))
@@ -2428,6 +2530,12 @@ static void mainwindow_init(Mainwindow *obj)
 
      if (!geometry_stack_pop(&window_geometry_stack,NULL,GTK_WINDOW(obj)))
 	  gtk_window_set_default_size(GTK_WINDOW(obj),540,230);
+
+     /* Setup drag'n'drop target */
+     gte.target = "text/uri-list";
+     gte.flags = gte.info = 0;
+     gtk_drag_dest_set(GTK_WIDGET(obj), GTK_DEST_DEFAULT_ALL, &gte, 1, 
+		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
 }
 
 guint mainwindow_get_type(void)
